import asyncio
import base64
import random
import re
from io import BytesIO
from math import ceil
from pathlib import Path
from typing import List, Literal, Optional, Tuple, Union

from PIL import Image, ImageDraw, ImageFile, ImageFilter, ImageFont
from nonebot import logger

ImageFile.LOAD_TRUNCATED_IMAGES = True
Image.MAX_IMAGE_PIXELS = None


class BuildImage:
    """
    快捷生成图片与操作图片的工具类
    """

    def __init__(
        self,
        w: int,
        h: int,
        paste_image_width: int = 0,
        paste_image_height: int = 0,
        color: Union[str, Tuple[int, int, int], Tuple[int, int, int, int]] = None,
        image_mode: str = "RGBA",
        font_size: int = 10,
        background: Union[Optional[str], BytesIO, Path] = None,
        font: str = "yz.ttf",
        ratio: float = 1,
        is_alpha: bool = False,
        plain_text: Optional[str] = None,
        font_color: Optional[Union[str, Tuple[int, int, int]]] = None,
    ):
        """
        参数:
            :param w: 自定义图片的宽度，w=0时为图片原本宽度
            :param h: 自定义图片的高度，h=0时为图片原本高度
            :param paste_image_width: 当图片做为背景图时，设置贴图的宽度，用于贴图自动换行
            :param paste_image_height: 当图片做为背景图时，设置贴图的高度，用于贴图自动换行
            :param color: 生成图片的颜色
            :param image_mode: 图片的类型
            :param font_size: 文字大小
            :param background: 打开图片的路径
            :param font: 字体，默认在 resource/ttf/ 路径下
            :param ratio: 倍率压缩
            :param is_alpha: 是否背景透明
            :param plain_text: 纯文字文本
        """
        self.w = int(w)
        self.h = int(h)
        self.paste_image_width = int(paste_image_width)
        self.paste_image_height = int(paste_image_height)
        self._current_w = 0
        self._current_h = 0
        self.font = ImageFont.truetype(str(FONT_PATH / font), int(font_size))
        if not plain_text and not color:
            color = (255, 255, 255)
        self.background = background
        if not background:
            if plain_text:
                if not color:
                    color = (255, 255, 255, 0)
                ttf_w, ttf_h = self.getsize(plain_text)
                self.w = self.w if self.w > ttf_w else ttf_w
                self.h = self.h if self.h > ttf_h else ttf_h
            self.markImg = Image.new(image_mode, (self.w, self.h), color)
            self.markImg.convert(image_mode)
        else:
            if not w and not h:
                self.markImg = Image.open(background)
                w, h = self.markImg.size
                if ratio and ratio > 0 and ratio != 1:
                    self.w = int(ratio * w)
                    self.h = int(ratio * h)
                    self.markImg = self.markImg.resize(
                        (self.w, self.h), Image.ANTIALIAS
                    )
                else:
                    self.w = w
                    self.h = h
            else:
                self.markImg = Image.open(background).resize(
                    (self.w, self.h), Image.ANTIALIAS
                )
        if is_alpha:
            try:
                array = self.markImg.load()
                for i in range(w):
                    for j in range(h):
                        pos = array[i, j]
                        is_edit = sum([1 for x in pos[0:3] if x > 240]) == 3
                        if is_edit:
                            array[i, j] = (255, 255, 255, 0)
            except Exception as e:
                logger.warning(f"背景透明化发生错误..{type(e)}：{e}")
        self.draw = ImageDraw.Draw(self.markImg)
        self.size = self.w, self.h
        if plain_text:
            fill = font_color if font_color else (0, 0, 0)
            self.text((0, 0), plain_text, fill)
        try:
            self.loop = asyncio.get_event_loop()
        except RuntimeError:
            new_loop = asyncio.new_event_loop()
            asyncio.set_event_loop(new_loop)
            self.loop = asyncio.get_event_loop()


async def text2image(
    text: str,
    auto_parse: bool = True,
    font_size: int = 20,
    color: Union[str, Tuple[int, int, int], Tuple[int, int, int, int]] = "white",
    font: str = "CJGaoDeGuo.otf",
    font_color: Union[str, Tuple[int, int, int]] = "black",
    padding: Union[int, Tuple[int, int, int, int]] = 0,
    _add_height: float = 0,
) -> BuildImage:
    """
    说明:
        解析文本并转为图片
        使用标签
            <f> </f>
        可选配置项
            font: str -> 特殊文本字体
            fs / font_size: int -> 特殊文本大小
            fc / font_color: Union[str, Tuple[int, int, int]] -> 特殊文本颜色
        示例
            在不在，<f font=YSHaoShenTi-2.ttf font_size=30 font_color=red>HibiKi小姐</f>，
            你最近还好吗，<f font_size=15 font_color=black>我非常想你</f>，这段时间我非常不好过，
            <f font_size=25>抽卡抽不到金色</f>，这让我很痛苦
    参数:
        :param text: 文本
        :param auto_parse: 是否自动解析，否则原样发送
        :param font_size: 普通字体大小
        :param color: 背景颜色
        :param font: 普通字体
        :param font_color: 普通字体颜色
        :param padding: 文本外边距，元组类型时为 （上，左，下，右）
        :param _add_height: 由于get_size无法返回正确的高度，采用手动方式额外添加高度
    """
    pw = ph = top_padding = left_padding = 0
    if padding:
        if isinstance(padding, int):
            pw = padding * 2
            ph = padding * 2
            top_padding = left_padding = padding
        elif isinstance(padding, tuple):
            pw = padding[0] + padding[2]
            ph = padding[1] + padding[3]
            top_padding = padding[0]
            left_padding = padding[1]
    if auto_parse and re.search(r"<f(.*)>(.*)</f>", text):
        _data = []
        new_text = ""
        placeholder_index = 0
        for s in text.split("</f>"):
            r = re.search(r"<f(.*)>(.*)", s)
            if r:
                start, end = r.span()
                if start != 0 and (t := s[:start]):
                    new_text += t
                _data.append(
                    [
                        (start, end),
                        f"[placeholder_{placeholder_index}]",
                        r.group(1).strip(),
                        r.group(2),
                    ]
                )
                new_text += f"[placeholder_{placeholder_index}]"
                placeholder_index += 1
        new_text += text.split("</f>")[-1]
        image_list = []
        current_placeholder_index = 0
        # 切分换行，每行为单张图片
        for s in new_text.split("\n"):
            _tmp_text = s
            img_height = BuildImage(0, 0, font_size=font_size).getsize("正")[1]
            img_width = 0
            _tmp_index = current_placeholder_index
            for _ in range(s.count("[placeholder_")):
                placeholder = _data[_tmp_index]
                if "font_size" in placeholder[2]:
                    r = re.search(r"font_size=['\"]?(\d+)", placeholder[2])
                    if r:
                        w, h = BuildImage(0, 0, font_size=int(r.group(1))).getsize(
                            placeholder[3]
                        )
                        img_height = img_height if img_height > h else h
                        img_width += w
                else:
                    img_width += BuildImage(0, 0, font_size=font_size).getsize(
                        placeholder[3]
                    )[0]
                _tmp_text = _tmp_text.replace(f"[placeholder_{_tmp_index}]", "")
                _tmp_index += 1
            img_width += BuildImage(0, 0, font_size=font_size).getsize(_tmp_text)[0]
            # img_width += len(_tmp_text) * font_size
            # 开始画图
            A = BuildImage(
                img_width, img_height, color=color, font=font, font_size=font_size
            )
            basic_font_h = A.getsize("正")[1]
            current_width = 0
            # 遍历占位符
            for _ in range(s.count("[placeholder_")):
                if not s.startswith(f"[placeholder_{current_placeholder_index}]"):
                    slice_ = s.split(f"[placeholder_{current_placeholder_index}]")
                    await A.atext(
                        (current_width, A.h - basic_font_h - 1), slice_[0], font_color
                    )
                    current_width += A.getsize(slice_[0])[0]
                placeholder = _data[current_placeholder_index]
                # 解析配置
                _font = font
                _font_size = font_size
                _font_color = font_color
                for e in placeholder[2].split():
                    if e.startswith("font="):
                        _font = e.split("=")[-1]
                    if e.startswith("font_size=") or e.startswith("fs="):
                        _font_size = int(e.split("=")[-1])
                        if _font_size > 1000:
                            _font_size = 1000
                        if _font_size < 1:
                            _font_size = 1
                    if e.startswith("font_color") or e.startswith("fc="):
                        _font_color = e.split("=")[-1]
                text_img = BuildImage(
                    0,
                    0,
                    plain_text=placeholder[3],
                    font_size=_font_size,
                    font_color=_font_color,
                    font=_font,
                )
                _img_h = (
                    int(A.h / 2 - text_img.h / 2)
                    if new_text == "[placeholder_0]"
                    else A.h - text_img.h
                )
                await A.apaste(text_img, (current_width, _img_h - 1), True)
                current_width += text_img.w
                s = s[
                    s.index(f"[placeholder_{current_placeholder_index}]")
                    + len(f"[placeholder_{current_placeholder_index}]") :
                ]
                current_placeholder_index += 1
            if s:
                slice_ = s.split(f"[placeholder_{current_placeholder_index}]")
                await A.atext((current_width, A.h - basic_font_h), slice_[0])
                current_width += A.getsize(slice_[0])[0]
            A.crop((0, 0, current_width, A.h))
            # A.show()
            image_list.append(A)
        height = 0
        width = 0
        for img in image_list:
            height += img.h
            width = width if width > img.w else img.w
        width += pw
        height += ph
        A = BuildImage(width + left_padding, height + top_padding, color=color)
        current_height = top_padding
        for img in image_list:
            await A.apaste(img, (left_padding, current_height), True)
            current_height += img.h
    else:
        width = 0
        height = 0
        _tmp = BuildImage(0, 0, font=font, font_size=font_size)
        _, h = _tmp.getsize("正")
        line_height = int(font_size / 3)
        image_list = []
        for x in text.split("\n"):
            w, _ = _tmp.getsize(x.strip() or "正")
            height += h + line_height
            width = width if width > w else w
            image_list.append(BuildImage(w, h, font=font, font_size=font_size, plain_text=x.strip(), color=color))
        width += pw
        height += ph
        A = BuildImage(
            width + left_padding,
            height + top_padding + 2,
            color=color,
        )
        cur_h = ph
        for img in image_list:
            await A.apaste(img, (pw, cur_h), True)
            cur_h += img.h + line_height
    return A


if __name__ == "__main__":
    pass



from enum import Enum, IntEnum


# 触发器：buff触发的时机
class Trigger(Enum):
    ACTIVE = 0  # 某个触发器触发
    ATTACK = 1  # 攻击
    # BE_ATTACK = 2  # 被攻击
    GET_ATK = 3  # 计算攻击时
    GET_DEF = 4  # 计算防御时
    GET_SPD = 5  # 计算速度时
    GET_HP = 6  # 计算生命时
    TURN_END = 7  # 回合结束时
    GET_LIFE_INC_SPD = 8  # 计算生命回复速度时
    DEAL_DAMAGE = 9  # 即将造成伤害（约定如果是体力流失，owner是空的）
    TAKEN_DAMAGE = 10  # 受到伤害（包括体力流失，一般用于计算死亡事件）
    GET_LV = 11  # 计算等级时
    GET_CRIT = 12  # 计算暴击率时
    GET_CSD = 13  # 计算暴击伤害时
    COUNTER = 14  # 反击
    TURN_START = 15  # 回合开始时


class DamageType(Enum):
    NORMAL = 0  # 普通伤害
    POISON = 1  # 毒伤害
    BURN = 2  # 燃烧伤害
    REAL = 3  # 真实伤害


class BuffTag(Enum):
    POISON_DEBUFF = 0  # 中毒类


class BuffPriority(IntEnum):
    ENVIRONMENT = 0  # 环境因子，发动非常早
    NORMAL = 1
    CHANGE_ATK_FIRST = 0
    CHANGE_ATK_NORMAL = 1  # 修改攻击力，标准优先级
    CHANGE_ATK_LAST = 2  # 修改攻击力，最后优先级
    CHANGE_DAMAGE_LAST = 2  # 修改伤害，最后优先级
    ABS_FINAL = 999  # 最后的最后才能运行，死亡判定专属优先级

    # 响应扭曲效果结算
    AVOID = 0  # 闪避最先结算

    # 回合结束结算
    TURN_END_LAST = 9



from ..psql_db import db

from datetime import datetime



# 其他杂项(热度图，签到等）
# 随便填了一些内容，方便更新
class OhterDB(db.Model):
    __tablename__ = "ikun_other"

    id: int = db.Column(db.Integer(), primary_key=True)
    # 配置名称
    name = db.Column(db.String(), nullable=False)
    # 配置类型
    type = db.Column(db.String(), nullable=False)
    # uid
    uid = db.Column(db.String(), nullable=True)
    # 时间
    start_time = db.Column(db.DateTime(), default=datetime.now, nullable=True)
    # 计数
    count = db.Column(db.Integer(), default=0, nullable=True)
    # 信息
    info: str = db.Column(db.JSON(), default={}, nullable=True)

    # @classmethod
    # async def add_new(cls, uid: str, name: str, times: int) -> "OhterDB":
    #     # 这里要计算一下是第几个获得的，其实业务逻辑不应该写在这但是我懒了，一个事务完成
    #     try:
    #         async with db.transaction():
    #             return await cls.create(uid=uid, group=name, order=1 + num, times=times)
    #     except Exception as e:
    #         logger.info(f"新建成就出错 {type(e)}: {e}")



import io
import os
from pathlib import Path
from typing import Union

from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageSegment, Bot, GroupMessageEvent



def add_items(a, b) -> dict[str, int]:
    temp = dict()
    for key in a.keys() | b.keys():
        temp[key] = round(sum([d.get(key, 0) for d in (a, b)]), 2)
    return temp


def add_item(dic, name, cnt=1):
    if dic.get(name) is None:
        dic[name] = round(cnt)
    else:
        dic[name] = round(dic[name] + cnt, 2)


game_path = os.path.dirname(__file__)


# 不知道官方的那个image怎么用 只能直接用自己文件下的的resources+MessageSegment了 会用的可以告诉我
def get_image(type, name):
    image_file = f"file:///{game_path}/gamedata/image/{type}/{name}"
    return MessageSegment.image(image_file)


SESSION_CODE = "S1"  # 不同赛季的账号是完全不同的


def get_uid(event) -> str:
    return str(event.user_id) + SESSION_CODE


def get_act_str(name) -> str:
    return '正在' + name


async def send_group_msg(
        bot: Bot,
        event: GroupMessageEvent,
        name: str,
        msgs: list[str],
):
    """
    发送合并消息(发送人名称相同)
    @param bot: 机器人的引用
    @param event: 用来获取群id
    @param name: 发消息的人的名字
    @param msgs: 要发的消息(list[str])
    @return:
    """
    messages = [MessageSegment.node_custom(bot.self_id, name, m) for m in msgs]
    await bot.call_api(
        "send_group_forward_msg", group_id=event.group_id, messages=messages
    )


async def send_group_msg_pic(
        bot: Bot,
        name: str,
        msgs: list[str],
):
    """
    发送合并消息(发送人名称相同)的图片，风控时使用
    @param bot: 机器人的引用
    @param event: 用来获取群id
    @param name: 发消息的人的名字
    @param msgs: 要发的消息(list[str])
    @return:
    """
    msg = "\n\n".join(msgs)
    result = f"{name}:\n{msg}"
    await send_img(bot, result)


async def send_group_msg_pic2(
        bot: Bot,
        msgs: list[tuple],
):
    """
    发送合并消息(发送人名称不同)的图片，风控时使用
    @param bot: 机器人的引用
    @param event: 用来获取群id
    @param msgs: 要发的消息([list[(str,str)]）e.g.[("真寻","我爱你"),...]
    @return:
    """
    tmp = io.StringIO()
    tmp2 = ""
    for name, msg in msgs:
        if name != tmp2:
            tmp2 = name
            tmp.write(name)
            tmp.write(':\n')
        tmp.write(msg)
        tmp.write('\n')
    await send_img(bot, tmp.getvalue())


# text2image是真寻独有的方法 nonebot显示不了:(
async def send_img(bot, msg: str):
    return


async def send_group_msg2(
        bot: Bot,
        event: GroupMessageEvent,
        msgs: list[tuple],
):
    """
    发送合并消息(发送人名称不同)
    @param bot: 机器人的引用
    @param event: 用来获取群id
    @param msgs: 要发的消息([list[(str,str)]）e.g.[("真寻","我爱你"),...]
    @return:
    """
    messages = [MessageSegment.node_custom(bot.self_id, m[0], m[1]) for m in msgs]
    await bot.call_api(
        "send_group_forward_msg", group_id=event.group_id, messages=messages
    )


def fill_list(my_list: list, length, fill=None):  # 使用 fill字符/数字 填充，使得最后的长度为 length
    if len(my_list) >= length:
        return my_list
    else:
        return my_list + (length - len(my_list)) * [fill]


def image(
        file: Union[str, Path, bytes] = None,
        path: str = None,
        b64: str = None,
) -> Union[MessageSegment, str]:
    """
    说明:
        生成一个 MessageSegment.image 消息
        生成顺序：绝对路径(abspath) > base64(b64) > img_name
    参数:
        :param file: 图片文件名称，默认在 resource/img 目录下
        :param path: 图片所在路径，默认在 resource/img 目录下
        :param b64: 图片base64
    """
    if isinstance(file, Path):
        if file.exists():
            return MessageSegment.image(file)
        logger.warning(f"图片 {file.absolute()}缺失...")
        return ""
    elif isinstance(file, (bytes, io.BytesIO)):
        return MessageSegment.image(file)
    elif b64:
        return MessageSegment.image(b64 if "base64://" in b64 else "base64://" + b64)
    else:
        if file.startswith("http"):
            return MessageSegment.image(file)
        if len(file.split(".")) == 1:
            file += ".jpg"
        else:
            logger.warning(f"图片 {file} 缺失...")
            return ""



import unittest

from src.gameBoard import GameBoard
from src.logger import Logger
from src.msgManager import MsgManager
from src.pokemon import Pokemon
from src.tools.tools import get_container


class TestCases(unittest.TestCase):
    container = None
    logger = None
    msgManager = None
    gameBoard = None

    @classmethod
    def setUpClass(cls) -> None:
        # print('每个用例前置')
        cls.container = get_container()
        # 单例声明
        cls.logger = cls.container[Logger] = Logger()
        cls.msgManager = cls.container[MsgManager] = MsgManager(cls.container[Logger])
        cls.gameBoard = cls.container[GameBoard] = GameBoard(cls.container[Logger], cls.container[MsgManager])

    def setUp(self) -> None:
        self.gameBoard = self.container[GameBoard]
        self.gameBoard.TURN_LIMIT =7
        self.pkm1 = self.container[Pokemon]
        self.pkm2 = self.container[Pokemon]

        data_init(self.pkm1)
        self.pkm1.name = "小白菜"
        data_init(self.pkm2)
        self.pkm2.name = "小黄瓜"

        self.gameBoard.add_ally(self.pkm1)
        self.gameBoard.add_enemy(self.pkm2)

    def tearDown(self) -> None:
        # print('每个用例的后置')
        self.gameBoard.print_log()
        self.msgManager.clean()
        self.logger.clean()

        if self.result == '我方胜利':
            print(f"{self.pkm1.name}胜利")
        else:
            print(f"{self.pkm1.name}战败！")

    def test发烟测试(self):
        self.pkm1.MAX_HP = 30
        self.pkm2.MAX_HP = 30
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 10)
        self.assertEqual(self.pkm2.hp, 0)

    def test连击(self):
        self.pkm1.skillGroup = ["连击"]

        # pkm2.skillGroup = ["反击50"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 1000)

    def test反击对连击正常反应(self):
        self.pkm1.skillGroup = ["连击"]
        self.pkm2.skillGroup = ["反击50"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 790)
        self.assertEqual(self.pkm2.hp, 1000)

    def test03(self):
        self.pkm2.MAX_HP = 100
        self.pkm2.skillGroup = ["愈合1"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 37)

    def test测试2个回血技能(self):
        self.pkm2.MAX_HP = 100
        self.pkm2.skillGroup = ["愈合1", "长生100"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 44)

    def test毒vs愈合(self):
        self.pkm2.MAX_HP = 100
        self.pkm2.skillGroup = ["愈合1", "长生100"]
        self.pkm1.skillGroup = ["毒液50"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, -2)

    def test毒攻击(self):
        self.pkm1.skillGroup = ["毒液50"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 870)

    def test毒攻击无法被反击(self):
        self.pkm1.skillGroup = ["毒液50", "连击", "尖角10"]
        self.pkm2.skillGroup = ["反击"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 870)

    def test连击会导致攻击力归零(self):
        self.pkm1.skillGroup = ["毒液50", "连击"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 1000)

    def test剧毒之体(self):
        self.pkm1.skillGroup = ["毒液50", "剧毒之体"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 844)

    def test剧毒之体无法造成普通伤害(self):
        self.pkm1.skillGroup = ["剧毒之体"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 1000)

    def test剧毒之体回合结束会强制解毒(self):
        self.pkm1.skillGroup = ["毒液50"]
        self.pkm2.skillGroup = ["剧毒之体"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 1000)
        self.assertEqual(self.pkm2.hp, 930)

    def test猛毒(self):
        self.pkm1.skillGroup = ["毒液50", "猛毒200"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 610)

    def test惜别(self):
        self.pkm1.skillGroup = ["惜别"]
        self.pkm1.MAX_HP = 10
        self.pkm2.skillGroup = ["惜别"]
        self.pkm2.MAX_HP = 10
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 1)
        self.assertEqual(self.pkm2.hp, -9)

    def test暴怒(self):
        self.pkm1.skillGroup = ["惜别", "暴怒1000"]
        self.pkm1.hp = 1000
        self.pkm2.skillGroup = ["惜别", "暴怒1000"]
        self.pkm2.hp = 1000
        self.gameBoard.init()
        self.pkm1.hp = 1
        self.pkm2.hp = 1
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 1)
        self.assertEqual(self.pkm2.hp, -1008)

    def test粘液(self):
        self.pkm1.skillGroup = ["粘液100"]
        self.pkm2.skillGroup = ["利爪100"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 1000)
        self.assertEqual(self.pkm2.hp, 930)

    def test抗毒(self):
        self.pkm1.skillGroup = ["毒液100"]
        self.pkm2.skillGroup = ["抗毒80"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 948)

    def test野性(self):
        self.pkm1.skillGroup = ["尖角10", "野性100"]
        self.pkm2.skillGroup = []
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 720)

    def test尖牙(self):
        self.pkm1.skillGroup = ["尖牙100"]
        self.pkm2.skillGroup = []
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 860)

    def test反击对反击不能触发无限反击(self):
        self.pkm1.skillGroup = ["反击100"]
        self.pkm2.skillGroup =["反击100"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 790)
        self.assertEqual(self.pkm2.hp, 790)

    def test两个人都有毒先手的人获胜(self):
        self.pkm1.skillGroup = ["毒液10000"]
        self.pkm2.skillGroup = ["毒液10000"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 1000)
        self.assertEqual(self.pkm2.hp, -1000)

    def test游泳测试1(self):
        self.pkm1.skillGroup = ["溺水"]
        self.pkm2.skillGroup = ["溺水"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 104)
        self.assertEqual(self.pkm2.hp, 104)

    def test免疫溺水(self):
        self.pkm1.skillGroup = ["溺水","游泳5"]
        self.pkm2.skillGroup = ["溺水"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 854)
        self.assertEqual(self.pkm2.hp, 91)

    def test平地游泳(self):
        self.pkm1.skillGroup = ["游泳5"]
        self.pkm2.skillGroup = []
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp, 930)

    def test水栖免疫溺水(self):
        self.pkm1.skillGroup = ["溺水","水栖"]
        self.pkm2.skillGroup = ["溺水"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 944)
        self.assertEqual(self.pkm2.hp, 90)

    def test灵巧能闪避毒(self):
        self.pkm1.skillGroup = ["灵巧100"]
        self.pkm2.skillGroup = ["毒液50"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 1000)
        self.assertEqual(self.pkm2.hp, 930)

    def test粘液降低攻击的优先级(self):
        self.pkm1.skillGroup = ["粘液50"]
        self.pkm2.skillGroup = ["尖角100"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 650)
        self.assertEqual(self.pkm2.hp, 930)

    def test毒刃被剧毒之体加成(self):
        self.pkm1.skillGroup = ["尖角90","毒刃40","剧毒之体"]
        self.pkm2.skillGroup = ["尖角100"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 450)
        self.assertEqual(self.pkm2.hp, -8)

    def test剧毒之体无效伤害(self):
        self.pkm1.skillGroup = ["尖角90","剧毒之体"]
        self.pkm2.skillGroup = ["尖角100"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 230)
        self.assertEqual(self.pkm2.hp, 1000)

    def test威压测试(self):
        self.pkm1.skillGroup = ["蛊神1","威压1"]
        self.pkm2.skillGroup = []
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 951)
        self.assertEqual(self.pkm2.hp,730)

    def test伤噬测试(self):
        self.pkm1.skillGroup = ["蛊神1","伤噬35","毒液100"]
        self.pkm2.skillGroup = []
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 930)
        self.assertEqual(self.pkm2.hp,558)


    def test镜子(self):
        self.pkm1.skillGroup = ["蛊神1","伤噬35","毒液100"]
        self.pkm2.skillGroup = ["镜中自我"]
        self.gameBoard.init()
        self.result = self.gameBoard.battle()
        self.assertEqual(self.pkm1.hp, 545)
        self.assertEqual(self.pkm2.hp,559)

if __name__ == '__main__':
    unittest.main()


def data_init(pkm: Pokemon):
    pkm.ATK = 20
    pkm.DEF = 10
    pkm.MAX_HP = 1000
    pkm.skillGroup = []



import os, tarfile

def make_targz(output_filename, source_dir):
    with tarfile.open(output_filename, "w:gz") as tar:
        tar.add(source_dir, arcname=os.path.basename(source_dir))


make_targz('gamedata/gamedata', 'gamedata/json/')




from ..psql_db import db

from datetime import datetime
from nonebot import logger



# 记录战斗日志 这个表默认为脏数据 不能参与任何业务流程
class BattleLogDB(db.Model):
    __tablename__ = "ikun_battle_log"

    id: int = db.Column(db.Integer(), primary_key=True)
    # 角色ID
    uid = db.Column(db.String(), nullable=False)
    # 创建时间
    start_time = db.Column(db.DateTime(), default=datetime.now)
    # 我方名字
    our_name: str = db.Column(db.String(), nullable=False)
    # 敌人名字
    enemy_name: str = db.Column(db.String(), nullable=False)
    our_lv: int = db.Column(db.Integer(), nullable=True)
    enemy_lv: int = db.Column(db.Integer(), nullable=True)
    # 对于单个玩家的战斗次数
    battle_times: int = db.Column(db.Integer(), nullable=False, default=0)
    # 日志
    log = db.Column(db.JSON(), nullable=False, default={})

    @classmethod
    async def record_battle(cls, uid: str, our_name: str, enemy_name: str,
                            our_lv: int, enemy_lv: int,
                            log: dict[str, str], battle_times: int
                            ) -> "BattleLogDB":

        try:
            async with db.transaction():
                return await cls.create(uid=uid, log=log, our_name=our_name, enemy_name=enemy_name, our_lv=our_lv,
                                        enemy_lv=enemy_lv, battle_times=battle_times)
        except Exception as e:
            logger.error(f"存储战斗日志出错 {type(e)}: {e}")

    @classmethod
    async def query_by_id(cls, qid: int) -> "BattleLogDB":
        try:
            async with db.transaction():
                log = await cls.query.where(cls.id == qid).gino.first()
                if log is None:
                    logger.warning(f"查询战斗日志出错 查询id为{id}")
                return log
        except Exception as e:
            logger.error(f"查询战斗日志出错 {type(e)}: {e}")

    @classmethod
    async def query_log_count(cls, uid: str) -> int:
        try:
            async with db.transaction():
                num = await db.select([db.func.count(BattleLogDB.id)]) \
                    .where(cls.uid == uid).gino.scalar()
                return num
        except Exception as e:
            logger.error(f"查询战斗日志出错 {type(e)}: {e}")



from typing import Callable, Any


from .logger import Logger
from .msgPack import MsgPack
from .tools.tools import get_container
from .enums import Trigger, BuffPriority


class Buff:

    def __init__(self, logger: Logger) -> None:
        self.logger = logger
        self._name = "未定义"  # buff的名字
        self.anti = []  # 这个buff施加后，这里面的buff将无法施加，存储内容为buff的名字
        self.trigger: Trigger = None  # buff施加的时机
        self._time = 9999  # buff持续的时间，9999为无限
        self._owner = None  # 拥有者
        self._tag = []  # 这个Buff拥有的标签
        self._del_msg = None  # 删除标签时的提示
        self._checker: list[Callable[[MsgPack], bool]] = []  # 这里传入一个函数，函数为真值则继续执行
        self._priority = BuffPriority.NORMAL  # buff优先级，基本都是1
        self.disable = False  # True时无法触发

        def default_handler(pack):
            self.logger.log(f"触发了{self._name}的默认行为")

        self._handler: Callable[[MsgPack], None] = default_handler

    def __str__(self) -> str:
        return f"【{self._owner}_{self._name}_{self.trigger}】"

    # def default_checker(self, pack):
    #     return True

    def name(self, name: str) -> "Buff":
        self._name = name
        return self

    def priority(self, priority) -> "Buff":
        self._priority = priority
        return self

    def get_priority(self):
        return self._priority

    def checker(self, checker: Callable[[Any], bool]) -> "Buff":
        self._checker.append(checker)
        return self

    def handler(self, handler: Callable[[MsgPack], None]) -> "Buff":
        self._handler = handler
        return self

    def check(self, pack: MsgPack) -> bool:
        if self.disable:
            return False
        pack.buff_name(self._name)
        pack.buff_owner(self._owner)
        for c in self._checker:
            if not c(pack):
                return False
        return True

    def handle(self, pack) -> None:
        pack.buff_name(self._name)
        return self._handler(pack)

    def time(self, time: int) -> "Buff":
        self._time = time
        return self

    def get_time(self):
        return self._time

    def time_pass(self):
        self._time -= 1

    def mark_as_delete(self) -> "Buff":
        self._time = -1
        return self

    def mark_as_disable(self):
        self.disable = True

    def owner(self, owner) -> "Buff":
        self._owner = owner
        return self

    def tag(self, tag) -> "Buff":
        self._tag.append(tag)
        return self

    def check_tag(self, tag):
        return tag in self._tag

    def check_owner(self, owner):
        return self._owner == owner

    def get_del_msg(self):
        return self._del_msg


def new_buff(owner: "Pokemon", trigger: Trigger) -> "Buff":
    tmp = get_container()[Buff]
    tmp.trigger = trigger
    tmp._owner = owner
    return tmp



from .logger import Logger
from .msgManager import MsgManager
from .msgPack import MsgPack
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .pokemon import Pokemon


class GameBoard:
    def __init__(self, logger: Logger, msg_manager: MsgManager):
        self.enemy:"Pokemon" = None
        self.our:"Pokemon" = None
        self.msg_manager = msg_manager
        self.logger = logger
        self.TURN_LIMIT = 20

    def add_ally(self, pokemon: "Pokemon"):
        self.our = pokemon

    def add_enemy(self, pokemon: "Pokemon"):
        self.enemy = pokemon

    def init(self):
        # 这个极端特殊的技能，暂且放在外面
        if "镜中自我" in self.enemy.skillGroup:
            self.enemy.ATK = self.our.ATK
            self.enemy.DEF = self.our.DEF
            self.enemy.MAX_HP = self.our.MAX_HP
            self.enemy.SPD = self.our.SPD + 1
            self.enemy.baselv = self.our.baselv
            self.enemy.fakebaselv = self.our.fakebaselv
            self.enemy.lv = self.our.lv
            self.enemy.skillGroup = self.our.skillGroup

        self.our.init()
        self.our.party = "our"

        self.enemy.init()
        self.enemy.party = "enemy"


    def get_enemy(self) -> "Pokemon":
        return self.enemy

    def battle(self):
        our: "Pokemon" = self.our
        enemy: "Pokemon" = self.enemy
        cnt = 1
        self.logger.log(
            f"{our.name}(lv.{our.get_lv()} 攻:{our.get_atk()} 防:{our.get_def()} 生命:{our.get_max_hp()} 速度:{our.get_spd()})")
        self.logger.log(
            f"{enemy.name}(lv.{enemy.get_lv()} 攻:{enemy.get_atk()} 防:{enemy.get_def()} 生命:{enemy.get_max_hp()} 速度:{enemy.get_spd()})")
        self.logger.log(f"战斗开始")

        while True:
            self.logger.log(f"🔚🔴第{cnt}回合")
            self.msg_manager.send_msg(MsgPack.turn_start_pack(cnt))
            # 第一步比较速度
            if our.get_spd() >= enemy.get_spd():
                our.attack(enemy)
                result = self.death_check(our, enemy)
                if result:
                    return result
                enemy.attack(our)
                result = self.death_check(our, enemy)
                if result:
                    return result
            else:
                enemy.attack(our)
                result = self.death_check(our, enemy)
                if result:
                    return result
                our.attack(enemy)
                result = self.death_check(our, enemy)
                if result:
                    return result
            self.msg_manager.send_msg(MsgPack.turn_end_pack())
            self.msg_manager.turn_end()
            result = self.death_check(our, enemy)
            if result:
                return result
            cnt += 1
            if cnt > self.TURN_LIMIT:
                self.logger.log(f"{self.our.name}和{self.enemy.name}永远战斗在了一起...")
                return "敌人胜利"

    def death_check(self, our: "Pokemon", enemy: "Pokemon") -> str:
        # 死亡结算
        if our.hp <= 0:
            self.logger.log(f"{self.our.name}倒下了...")
            return "敌人胜利"
        if enemy.hp <= 0:
            self.logger.log(f"{self.our.name}一口将{self.enemy.name}吞掉！")
            return "我方胜利"
        return None

    def print_log(self):
        self.logger.print_log()



from nonebot.adapters.onebot.v11 import GroupMessageEvent

from .player_handler import get_player
from .game_handler import get_world_data
from ..model.player_model import PlayerDB
from .game_handler import Item


async def get_store_list(event: GroupMessageEvent) -> [dict[str, Item], str]:
    tmp = get_world_data().get_shop_item()
    player = await get_player(event)
    money = player.query_item('金币')
    return tmp, convert_shop_item_to_str(tmp, money)


def convert_shop_item_to_str(tmp, money) -> str:
    ans = f"商品列表：  当前点数{money}\n0: 我不买了\n"
    i = 1
    for item in tmp:
        ans += f"{i}: {item.name}: {item.cost}点\n"
        i += 1
    return ans


async def buy_item_handle(event, itemname, itemcost, itemcnt) -> str:
    player:PlayerDB = await get_player(event)
    dif = itemcnt * itemcost - player.query_item("金币")
    if dif > 0:
        return f"你买不起这么贵的商品，还差{dif}点，点数可以打工获得"
    player.add_item(itemname, itemcnt)
    player.cost_item('金币', dif)
    await player.update(bag=player.bag,collection=player.collection).apply()
    return f"从老板那里获得了{itemname} x{itemcnt}"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_ikun_evolution',
 'nonebot_plugin_ikun_evolution.model',
 'nonebot_plugin_ikun_evolution.psql_db',
 'nonebot_plugin_ikun_evolution.service',
 'nonebot_plugin_ikun_evolution.xiaor_battle_system',
 'nonebot_plugin_ikun_evolution.xiaor_battle_system.src',
 'nonebot_plugin_ikun_evolution.xiaor_battle_system.src.tools']

package_data = \
{'': ['*'],
 'nonebot_plugin_ikun_evolution': ['gamedata/*',
                                   'gamedata/image/item/*',
                                   'gamedata/image/notice/*']}

install_requires = \
['gino>=1.0.1,<2.0.0',
 'json5>=0.9.10,<0.10.0',
 'lagom>=2.2.0,<3.0.0',
 'nonebot-adapter-onebot>=2.0.0-beta.1,<3.0.0',
 'nonebot2>=2.0.0-beta.1,<3.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-ikun-evolution',
    'version': '1.0.2',
    'description': '移植自真寻的q群小游戏',
    'long_description': '# 只因进化录\n\n移植自真寻的[只因进化录](https://github.com/RShock/ikun_evolution)\n\n## 安装\n\n### 插件安装\n```\npip install nonebot_plugin_ikun_evolution\n```\n在 nonebot2 项目中设置 load_plugin()\n```\nnonebot.load_plugin(\'nonebot_plugin_ikun_evolution\')\n```\n\n### 数据库配置\n\n你需要安装一个[postgresql数据库](https://hibikier.github.io/zhenxun_bot/docs/installation_doc/install_postgresql.html)才能进行游戏\n\n安装完毕后，在`env.dev`里填上刚刚的数据库链接\n```\npsql = "postgresql://名字:密码@127.0.0.1:5432/数据库名字"\n```\n\n如果按真寻教程就是\n```\npsql = "postgresql://uname:zhenxun@127.0.0.1:5432/testdb"\n```\n\n这步有点困难，好处是再也不用担心误删数据库了。请加油罢\n',
    'author': '小r',
    'author_email': '418648118@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/RShock/nonebot_plugin_ikun_evolution',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import math

from ..model.battle_log_model import BattleLogDB
from ..model.player_model import PlayerDB
from .game_handler import get_world_data, Monster
from ..utils import get_uid
from ..xiaor_battle_system.src.gameBoard import GameBoard
from ..xiaor_battle_system.src.logger import Logger
from ..xiaor_battle_system.src.msgManager import MsgManager
from ..xiaor_battle_system.src.pokemon import Pokemon
from ..xiaor_battle_system.src.tools.tools import get_container

container = get_container()
# 单例声明
logger = container[Logger] = Logger()
msgManager = container[MsgManager] = MsgManager(container[Logger])
gameBoard = container[GameBoard] = GameBoard(container[Logger], container[MsgManager])


# 调用战斗模块 不知道性能开销有多大...如果太大我再优化一下
def get_player_skill(player, param: list[str]):
    skill = []
    for name in param:
        if name is None:
            continue
        skill.append(name + str(math.floor(int(player.skill.get(name)))))
    return skill


def battle(data: dict, player: PlayerDB, enemy: Monster):
    msgManager.clean()
    logger.clean()
    pkm1 = container[Pokemon]
    pkm2 = container[Pokemon]

    # 升级就是双方各自乘以1.1倍的数据，和每升级可谓是毫无区别，但是想必会给玩家自己变强的错觉罢
    pkm1.name = player.name
    pkm1.MAX_HP = data["hp"]
    pkm1.ATK = data["atk"]
    pkm1.DEF = data["def"]
    pkm1.SPD = data["spd"]
    pkm1.lv = data["lv"]
    lv = data.get("base_lv", player.lv)
    pkm1.baselv = lv
    pkm1.fakebaselv = lv
    # 技能处理这里稍微复杂一点
    # 首先是玩家自带的技能
    pkm1.skillGroup = get_player_skill(player, data["skill"])
    # 然后是可能存在的地图技能（但是目前并不存在，不写了）
    # sp技能（也不存在，以后再写）
    if "游泳" not in data["skill"] and player.query_item("乌龟护符") >= 0.99:    # 本赛季没有装备系统，写个特判
        pkm1.skillGroup.append("游泳4")

    pkm2.name = enemy.name
    pkm2.MAX_HP = enemy.hp
    pkm2.ATK = enemy.atk
    pkm2.DEF = enemy.defe
    pkm2.SPD = enemy.spd
    pkm2.lv = enemy.lv
    pkm2.baselv = enemy.baselv
    pkm2.fakebaselv = 1
    pkm2.skillGroup = enemy.sp_skill + enemy.skill

    gameBoard.add_ally(pkm1)
    gameBoard.add_enemy(pkm2)
    gameBoard.init()
    result = gameBoard.battle()
    return result, logger.get_log()


async def query_battle_log(id: str):
    log: BattleLogDB = await BattleLogDB.query_by_id(int(id))
    if log is None:
        return '未查到该场战斗'
    times = log.battle_times if log.battle_times != 0 else '?'
    battle_log = log.log["log"].split("🔚")
    tmp = [("战报播放系统",
         f"{log.our_name}(等级{log.our_lv})的第{times}次战斗\n对手：{log.enemy_name}(等级{log.enemy_lv})\n")]
    for l in battle_log:
        tmp.append(("战斗详情", l))
    return tmp


# 查询一个角色战斗的次数
# 因为战斗日志表是脏表所以这个查询毫无意义，但是写了也不至于删掉
async def query_battle_times(uid):
    num: int = await BattleLogDB.query_log_count(uid)
    from nonebot.log import logger
    logger.info(f"{uid}我出征了{num}次")
    return num


async def foo():
    return
    players: list[PlayerDB] = await PlayerDB.get_all()
    for player in players:
        if "防御成长" in player.skill:
            lv = player.skill["防御成长"]
            if round(lv, 2) >= 1.2 * 1.2 * 6:
                player.skill["防御成长"] = 6
            else:
                lv = round(lv / 1.2 / 1.2, 2)
            player.skill["防御成长"] = lv
            await player.update(skill=player.skill).apply()


async def sim_battle(event, our_lv, enemy_lv, enemy_name):
    player = await PlayerDB.get_player_by_uid(get_uid(event))
    enemy = get_world_data().get_enemy(enemy_name)
    msgManager.clean()
    logger.clean()
    pkm1 = container[Pokemon]
    pkm2 = container[Pokemon]
    pkm1.name = player.name
    pkm1.MAX_HP = player.hp
    pkm1.ATK = player.atk
    pkm1.DEF = player.defe
    pkm1.SPD = player.spd
    pkm1.lv = int(our_lv)
    pkm1.baselv = player.lv
    pkm1.fakebaselv = player.lv
    # 技能处理这里稍微复杂一点
    # 首先是玩家自带的技能
    pkm1.skillGroup = get_player_skill(player, player.equip_skill)
    # 然后是可能存在的地图技能（但是目前并不存在，不写了）
    # sp技能（也不存在，以后再写）

    pkm2.name = enemy.name
    pkm2.MAX_HP = enemy.hp
    pkm2.ATK = enemy.atk
    pkm2.DEF = enemy.defe
    pkm2.SPD = enemy.spd
    pkm2.lv = int(enemy_lv)
    pkm2.baselv = enemy.baselv
    pkm2.fakebaselv = 1
    pkm2.skillGroup = enemy.sp_skill + enemy.skill

    gameBoard.add_ally(pkm1)
    gameBoard.add_enemy(pkm2)
    gameBoard.init()
    result = gameBoard.battle()
    return logger.get_log()



from ..utils import get_act_str
from datetime import datetime
from nonebot import logger
from ..psql_db import db



class ActionDB(db.Model):
    __tablename__ = "ikun_adventure"

    id: int = db.Column(db.Integer(), primary_key=True)
    # 角色ID
    uid = db.Column(db.String(), nullable=False)
    # 在干什么
    action = db.Column(db.String(), nullable=False)
    # 在哪里
    position = db.Column(db.String(), nullable=False)
    # 什么时候去的
    start_time = db.Column(db.DateTime(), default=datetime.now)
    # 懒更新时间
    update_time = db.Column(db.DateTime(), default=datetime.now)
    # 已经获得的物品
    item_get = db.Column(db.JSON(), nullable=False, default={})
    # 已经获得的技能经验
    skill_get = db.Column(db.JSON(), nullable=False, default={})
    # 日志
    log = db.Column(db.String(), nullable=False, default="")
    # 角色被附加的额外属性(记录角色受到的buff 角色的攻击一类 太多了不想写列了)
    other = db.Column(db.JSON(), nullable=False, default={})
    # 缺省字段
    reverse1 = db.Column(db.JSON(), nullable=True, default={})
    reverse2 = db.Column(db.JSON(), nullable=True, default={})
    reverse3 = db.Column(db.JSON(), nullable=True, default={})

    @classmethod
    async def go_outside(cls, uid: str, pos: str, other:dict, log) -> "ActionDB":
        try:
            async with db.transaction():
                logger.info(f"{uid}出发了")
                return await cls.create(uid=uid, position=pos, action=get_act_str(pos), log=log,
                                        other=other)
        except Exception as e:
            logger.info(f"角色出发出错 {type(e)}: {e}")

    @classmethod
    async def get_action_by_uid(cls, uid: str) -> "ActionDB":
        try:
            async with db.transaction():
                return await cls.query.where(cls.uid == uid).gino.first()
        except Exception as e:
            logger.info(f"角色查询出错 {type(e)}: {e}")



from nonebot.adapters.onebot.v11 import GroupMessageEvent

from .player_handler import get_player
from ..service import adv_handler
from ..service.adv_handler import finish_action, get_user_status
from .game_handler import get_world_data
from ..model.player_model import PlayerDB
from ..utils import get_act_str
from .game_handler import Item


async def get_usable_item(event: GroupMessageEvent) -> [dict[str, Item], str]:
    player: PlayerDB = await get_player(event)
    tmp = []
    tmp.append(("小沙漏", player.query_item("小沙漏")))
    for k, v in player.bag.items():
        if k != '小沙漏' and get_world_data().get_item(k).usable and v >= 1:
            tmp.append((k, v))
    return tmp, convert_usable_item_to_str(tmp)


def convert_usable_item_to_str(tmp) -> str:
    ans = "你要使用什么呢？\n0: 返回\n"
    i = 0
    for item in tmp:
        ans += f"{i + 1}: {item[0]} 数量 {item[1]}\n"
        i += 1
    return ans


async def use_item_handler(event, itemname, itemcnt) -> str:
    player, status, _ = await get_user_status(event)

    if itemname == '小沙漏':
        if player.buff.get('特殊任务中'):
            return '你正在特殊任务中，沙漏被禁用了'
        if status == '休息中':
            return '你还没出发呢！'
        cost = await adv_handler.should_speed_up_cost(event)
        if player.cost_item('小沙漏', cost):
            await player.update(bag=player.bag).apply()
        else:
            return f"需要{cost}个沙漏，你只有{player.query_item('小沙漏')}个"
        if await finish_action(event, lambda a: a.action != get_act_str('活动')):
            return f'用{cost}个沙漏完成了探索'
        else:
            return "故障，请通报管理员"
    if itemname == '大蟠桃':
        player.cost_item('大蟠桃', 1)
        await player.update(atk=player.atk + 20, defe=player.defe + 10, hp=player.hp + 100, lv=player.lv + 4,
                            bag=player.bag, spd=player.spd + 1).apply()
        return "吃下了猴王珍藏的桃子，你的等级居然永久提升了，实力劲增！(初始等级+4 攻+20 守+10 命+100 速+1）\n副作用：成长的基础等级+4，现在成长要在角色变成9级时才有原来5级的效果"
    # player.add_item(itemname, itemcnt)
    # await player.update(bag=player.bag, gold=player.gold + dif).apply()
    return f"未知错误，使用{itemname}失败"



from lagom import Container


def get_num(s: str) -> int:
    num = ''.join([x for x in s if x.isdigit()])
    return 0 if num == '' else int(num)


# lagom的container怎么不是单例模式？？
_container = None


def get_container():
    global _container
    if _container is None:
        _container = Container()
    return _container
